@@ -19,7 +19,7 @@
  *
  * input plugin for http network streams
  *
- * $Id: input_http.c,v 1.119 2006/06/20 01:46:41 dgp85 Exp $
+ * $Id: input_http.c,v 1.129 2007/03/17 16:47:16 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -176,6 +176,16 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
     return 1;
   }
   if (!info->h_name) return 1;
+  
+  if ( info->h_addr_list[0] ) {
+    /* \177\0\0\1 is the *octal* representation of 127.0.0.1 */
+    if ( info->h_addrtype == AF_INET && !memcmp(info->h_addr_list[0], "\177\0\0\1", 4) ) {
+      lprintf("host '%s' is localhost\n", host);
+      return 1;
+    }
+    /* TODO: IPv6 check */
+  }
+
   target = info->h_name;
 
   host_len = strlen(target);
@@ -325,12 +335,9 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
           /* prepares the event */
           radio = _x_meta_info_get(this->stream, XINE_META_INFO_ALBUM);
           if (radio) {
-            int len = strlen(radio);
-            strncpy(data.str, radio, sizeof(data.str));
-            strncat(data.str, " - ", sizeof(data.str) - len);
-            strncat(data.str, songtitle, sizeof(data.str) - len - 3);
+	    snprintf (data.str, sizeof(data.str), "%s - %s", radio, songtitle);
           } else {
-            strncpy(data.str, songtitle, sizeof(data.str));
+            strncpy(data.str, songtitle, sizeof(data.str)-1);
           }
           data.str[sizeof(data.str) - 1] = '\0';
           data.str_len = strlen(data.str) + 1;
@@ -356,7 +363,7 @@ static off_t http_plugin_read_int (http_input_plugin_t *this,
   int read_bytes = 0;
   int nlen;
   
-  lprintf("total=%lld\n", total);
+  lprintf("total=%"PRId64"\n", total);
   while (total) {
     nlen = total;
     if (this->shoutcast_mode &&
@@ -408,7 +415,7 @@ static off_t http_plugin_read (input_plugin_t *this_gen,
     else
       n = nlen;
 
-    lprintf ("%lld bytes from preview (which has %lld bytes)\n", n, this->preview_size);
+    lprintf ("%"PRId64" bytes from preview (which has %"PRId64" bytes)\n", n, this->preview_size);
     memcpy (buf, &this->preview[this->curpos], n);
 
     num_bytes += n;
@@ -567,7 +574,7 @@ static off_t http_plugin_seek(input_plugin_t *this_gen, off_t offset, int origin
   return this->curpos;
 }
 
-static char* http_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* http_plugin_get_mrl (input_plugin_t *this_gen) {
   http_input_plugin_t *this = (http_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -632,7 +639,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   http_input_class_t  *this_class = (http_input_class_t *) this->input_plugin.input_class;
   int                  done, len, linenum;
   int                  httpcode;
-  int                  res, progress;
+  int                  res;
   int                  buflen;
   int                  use_proxy;
   int                  proxyport;
@@ -665,7 +672,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   if (this->user && strlen(this->user)) {
     if (http_plugin_basicauth (this->user, this->password, this->auth, BUFSIZE)) {
       _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "basic auth error", NULL);
-      return 0;
+      return -1;
     }
   }
   
@@ -699,18 +706,30 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   this->curpos = 0;
   
   if (this->fh == -1)
-    return 0;
-  
-  /* connection timeout 20s */
-  progress = 0;
-  do {
-    report_progress(this->stream, progress);
-    res = _x_io_select (this->stream, this->fh, XIO_WRITE_READY, 500);
-    progress += 2;
-  } while ((res == XIO_TIMEOUT) && (progress < 100));
-  if (res != XIO_READY)
-    return 0;
-  
+    return -2;
+
+  {  
+    uint32_t         timeout, progress;
+    xine_cfg_entry_t cfgentry;
+    if (xine_config_lookup_entry (this->stream->xine, "media.network.timeout", &cfgentry)) {
+      timeout = cfgentry.num_value * 1000;
+    } else {
+      timeout = 30000; /* 30K msecs = 30 secs */
+    }
+
+    progress = 0;
+    do {
+      report_progress(this->stream, progress);
+      res = _x_io_select (this->stream, this->fh, XIO_WRITE_READY, 500);
+      progress += (500*100000)/timeout;
+    } while ((res == XIO_TIMEOUT) && (progress <= 100000));
+
+    if (res != XIO_READY) {
+      _x_message(this->stream, XINE_MSG_NETWORK_UNREACHABLE, this->mrl, NULL);
+      return -3;
+    }
+  }
+
   if (use_proxy) {
     if (this->port != DEFAULT_HTTP_PORT) {
       snprintf (this->buf, BUFSIZE, "GET http://%s:%d%s HTTP/1.0\015\012",
@@ -744,18 +763,16 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   }
   
   snprintf(this->buf + buflen, BUFSIZE - buflen,
-           "User-Agent: xine/%s\015\012", VERSION);
-  buflen = strlen(this->buf);
-  strncat (this->buf, "Accept: */*\015\012", BUFSIZE - buflen);
-  buflen = strlen(this->buf);
-  strncat (this->buf, "Icy-MetaData: 1\015\012", BUFSIZE - buflen);
-  buflen = strlen(this->buf);
-  strncat (this->buf, "\015\012", BUFSIZE - buflen);
+           "User-Agent: xine/%s\015\012"
+           "Accept: */*\015\012"
+           "Icy-MetaData: 1\015\012"
+           "\015\012",
+           VERSION);
   buflen = strlen(this->buf);
   if (_x_io_tcp_write (this->stream, this->fh, this->buf, buflen) != buflen) {
     _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "couldn't send request", NULL);
     xprintf(this_class->xine, XINE_VERBOSITY_DEBUG, "input_http: couldn't send request\n");
-    return 0;
+    return -4;
   }
 
   lprintf ("request sent: >%s<\n", this->buf);
@@ -765,13 +782,10 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   this->contentlength = 0;
 
   while (!done) {
-
-    /* 
-       printf ("input_http: read...\n");
-    */
+    /* fprintf (stderr, "input_http: read...\n"); */
 
     if (_x_io_tcp_read (this->stream, this->fh, &this->buf[len], 1) <= 0) {
-      return 0;
+      return -5;
     }
 
     if (this->buf[len] == '\012') {
@@ -803,7 +817,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	    _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "invalid http answer", NULL);
 	    xine_log (this->stream->xine, XINE_LOG_MSG, 
 		      _("input_http: invalid http answer\n"));
-	    return 0;
+	    return -6;
 	}
 
 	if (httpcode >= 300 && httpcode < 400) {
@@ -815,20 +829,20 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	  xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("input_http: http status not 2xx: >%d %s<\n"),
 		                        httpcode, httpstatus);
-	  return 0;
+	  return -7;
 	} else if (httpcode == 403) {
           _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
 	  xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("input_http: http status not 2xx: >%d %s<\n"),
 		    httpcode, httpstatus);
-	  return 0;
+	  return -8;
 	} else if (httpcode < 200 || httpcode >= 300) {
 	  _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "http status not 2xx: ",
 	               httpstatus, NULL);
       	  xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("input_http: http status not 2xx: >%d %s<\n"),
 		    httpcode, httpstatus);
-	  return 0;
+	  return -9;
 	}
       } else {
 	if (this->contentlength == 0) {
@@ -906,7 +920,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
        _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
        xine_log (this->stream->xine, XINE_LOG_MSG,
          _("input_http: buffer exhausted after %d bytes."), BUFSIZE);
-       return 0;
+       return -10;
     }
   }
 
@@ -918,7 +932,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   this->preview_size = MAX_PREVIEW_SIZE;
   if (this->is_nsv) {
     if (!resync_nsv(this))
-      return 0;
+      return -11;
     
     /* the first 3 chars are "NSV" */
     this->preview_size = http_plugin_read_int (this, this->preview + 3, MAX_PREVIEW_SIZE - 3);
@@ -927,10 +941,10 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   }
   if (this->preview_size < 0) {
     xine_log (this->stream->xine, XINE_LOG_MSG, _("input_http: read error %d\n"), errno);
-    return 0;
+    return -12;
   }
   
-  lprintf("preview_size=%lld\n", this->preview_size);
+  lprintf("preview_size=%"PRId64"\n", this->preview_size);
   this->curpos = 0;
   
   return 1;
@@ -978,7 +992,7 @@ static input_plugin_t *http_class_get_instance (input_class_t *cls_gen, xine_str
   return &this->input_plugin;
 }
 
-static char *http_class_get_description (input_class_t *this_gen) {
+static const char *http_class_get_description (input_class_t *this_gen) {
   return _("http input plugin");
 }
 
@@ -1078,7 +1092,7 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "http", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
