@@ -38,7 +38,7 @@
  * usage: 
  *   xine pvr:/<prefix_to_tmp_files>\!<prefix_to_saved_files>\!<max_page_age>
  *
- * $Id: input_pvr.c,v 1.62 2006/07/10 22:08:15 dgp85 Exp $
+ * $Id: input_pvr.c,v 1.65 2007/01/19 01:05:25 dgp85 Exp $
  */
 
 /**************************************************************************
@@ -1317,7 +1317,7 @@ static uint32_t pvr_plugin_get_blocksize (input_plugin_t *this_gen) {
   return PVR_BLOCK_SIZE;
 }
 
-static char* pvr_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* pvr_plugin_get_mrl (input_plugin_t *this_gen) {
   pvr_input_plugin_t *this = (pvr_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -1524,7 +1524,7 @@ static input_plugin_t *pvr_class_get_instance (input_class_t *cls_gen, xine_stre
  * plugin class functions
  */
 
-static char *pvr_class_get_description (input_class_t *this_gen) {
+static const char *pvr_class_get_description (input_class_t *this_gen) {
   return _("WinTV-PVR 250/350 input plugin");
 }
 
@@ -1548,9 +1548,9 @@ static void *init_plugin (xine_t *xine, void *data) {
   this->xine   = xine;
   this->config = xine->config;
 
-  this->devname = this->config->register_string(this->config,
+  this->devname = this->config->register_filename(this->config,
 				    "media.wintv_pvr.device",
-				    PVR_DEVICE,
+				    PVR_DEVICE, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 				    _("device used for WinTV-PVR 250/350 (pvr plugin)"),
 				    _("The path to the device of your WinTV card."),
 				    10, NULL,
