@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: ldt_keeper.c,v 1.15 2006/05/07 09:31:57 valtri Exp $
+ * $Id: ldt_keeper.c,v 1.16 2007/03/09 23:49:35 dgp85 Exp $
  *
  *
  * contents:
@@ -54,6 +54,7 @@
 
 /* applied some modification to make make our xine friend more happy */
 #include "ldt_keeper.h"
+#include "config.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -90,6 +91,7 @@ int modify_ldt(int func, void *ptr, unsigned long bytecount);
 #include <sys/sysi86.h>
 
 /* solaris x86: add missing prototype for sysi86() */
+#ifndef HAVE_SYSI86
 #ifdef  __cplusplus
 extern "C" {
 #endif
@@ -97,6 +99,7 @@ int sysi86(int, void*);
 #ifdef  __cplusplus
 }
 #endif
+#endif
 
 #ifndef NUMSYSLDTS             /* SunOS 2.5.1 does not define NUMSYSLDTS */
 #define NUMSYSLDTS     6       /* Let's hope the SunOS 5.8 value is OK */
