@@ -51,21 +51,53 @@
  * Bot 4 : Drop
  * Top 5 : Drop
  * Bot 5 : Show
+ *
+ *   A A   1
+ *   A B   2
+ *   B C   4
+ *   C C   8
+ *   D D   16
+ *
+ *  D D A A A B B C C C D D A A A +------------
+ * [       ]
+ * [* *  ]                        | 1    top      AA
+ *   [  * *]                      | 0 AA bottom   AA
+ *
+ *     [       ]
+ *     [* *  ]                    | 1    top      AB
+ *       [* *  ]                  | 1 AB bottom   AB
+ *
+ *         [       ]
+ *         [  * *]                | 0    top      BC
+ *           [* *  ]              | 1 BC bottom   BC 
+ *
+ *             [       ]
+ *             [  * *]            | 0    top      CC
+ *               [  * *]          | 0 CC bottom   CC 
+ *
+ *                 [       ]
+ *                 [* *  ]        | 1    top      DD
+ *                   [  * *]      | 0 DD bottom   DD 
+ *
+ *
+ *                     [* *  ]    | 1    top      AA
+ *                       [  * *]  | 0 AA bottom   AA
+ *
  */
 
-/* Offset                  1     2     3      4      5   */
-/* Field Pattern          [T B  T][B  T][B   T B]  [T B] */
-/* Action                 Copy  Save  Merge  Copy  Copy  */
-/*                              Bot   Top                */
-int tff_top_pattern[] = { 0,    1,    0,     0,    0     };
-int tff_bot_pattern[] = { 0,    0,    0,     1,    0     };
+/* Offset                               1     2     3      4      5   */
+/* Field Pattern                       [T B  T][B  T][B   T B]  [T B] */
+/* Action                              Copy  Save  Merge  Copy  Copy  */
+/*                                           Bot   Top                */
+static const int tff_top_pattern[] = { 0,    1,    0,     0,    0     };
+static const int tff_bot_pattern[] = { 0,    0,    0,     1,    0     };
 
-/* Offset                  1     2     3      4      5   */
-/* Field Pattern          [B T  B][T  B][T   B T]  [B T] */
-/* Action                 Copy  Save  Merge  Copy  Copy  */
-/*                              Top   Bot                */
-int bff_top_pattern[] = { 0,    0,    0,     1,    0     };
-int bff_bot_pattern[] = { 0,    1,    0,     0,    0     };
+/* Offset                               1     2     3      4      5   */
+/* Field Pattern                       [B T  B][T  B][T   B T]  [B T] */
+/* Action                              Copy  Save  Merge  Copy  Copy  */
+/*                                           Top   Bot                */
+static const int bff_top_pattern[] = { 0,    0,    0,     1,    0     };
+static const int bff_bot_pattern[] = { 0,    1,    0,     0,    0     };
 
 /* Timestamp mangling                                            */
 /* From the DVD :         0  +  3003+ 6006 + 9009+ 12012 = 15015 */
@@ -550,11 +582,19 @@ void diff_factor_packed422_frame( pulldown_metrics_t *peak, pulldown_metrics_t *
 
 int pulldown_source( int action, int bottom_field )
 {
-    if( action == PULLDOWN_SEQ_AB || action == PULLDOWN_SEQ_BC ) {
+    if( action == PULLDOWN_SEQ_AA ) {
+        return !bottom_field;
+    } else if( action == PULLDOWN_SEQ_AB ) {
+        return 1;
+    } else if( action == PULLDOWN_SEQ_BC ) {
         return bottom_field;
-    } else {
+    } else if( action == PULLDOWN_SEQ_CC ) {
+        return 0;
+    } else if( action == PULLDOWN_SEQ_DD ) {
         return !bottom_field;
     }
+
+    return 0;
 }
 
 int pulldown_drop( int action, int bottom_field )
