@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_goom.c,v 1.56.2.1 2005/07/17 21:50:06 miguelfreitas Exp $
+ * $Id: xine_goom.c,v 1.65 2007/02/20 00:58:51 dgp85 Exp $
  *
  * GOOM post plugin.
  *
@@ -43,14 +43,14 @@
 
 #include "goom.h"
 
-#define NUMSAMPLES  512
-#define FPS          10
+#define NUMSAMPLES  512 /* hardcoded into goom api */
+#define FPS          14
 
 #define GOOM_WIDTH  320
 #define GOOM_HEIGHT 240
 
 /* colorspace conversion methods */
-const char * goom_csc_methods[]={
+static const char const * goom_csc_methods[]={
   "Fast but not photorealistic",
   "Slow but looks better",
   NULL
@@ -83,7 +83,7 @@ struct post_plugin_goom_s {
   PluginInfo        *goom;
   
   int data_idx;
-  gint16 data [2][512];
+  gint16 data [2][NUMSAMPLES];
   audio_buffer_t buf;   /* dummy buffer just to hold a copy of audio data */
   
   int channels;
@@ -108,11 +108,11 @@ static void *goom_init_plugin(xine_t *xine, void *);
 
 
 /* plugin catalog information */
-post_info_t goom_special_info = { 
+static const post_info_t goom_special_info = { 
   XINE_POST_TYPE_AUDIO_VISUALIZATION
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_POST | PLUGIN_MUST_PRELOAD, 9, "goom", XINE_VERSION_CODE, &goom_special_info, &goom_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
@@ -218,7 +218,7 @@ static void *goom_init_plugin(xine_t *xine, void *data)
   
 
   cfg->register_enum (cfg, "effects.goom.csc_method", 0,
-                           (char **)goom_csc_methods,
+                           goom_csc_methods,
                            _("colorspace conversion method"),
                            _("You can choose the colorspace conversion method used by goom.\n"
 			     "The available selections should be self-explaining."),
@@ -297,7 +297,7 @@ static post_plugin_t *goom_open_plugin(post_class_t *class_gen, int inputs,
   outputv->xine_out.data   = (xine_video_port_t **)&this->vo_port;
   outputv->xine_out.rewire = goom_rewire_video;
   outputv->post            = &this->post;
-  xine_list_append_content(this->post.output, outputv);
+  xine_list_push_back(this->post.output, outputv);
   
   this->post.xine_post.audio_input[0] = &port->new_port;
 
@@ -318,6 +318,17 @@ static char *goom_get_description(post_class_t *class_gen)
 
 static void goom_class_dispose(post_class_t *class_gen)
 {
+  post_class_goom_t  *this = (post_class_goom_t*) class_gen;
+
+  this->xine->config->unregister_callback(this->xine->config,
+		  			  "effects.goom.fps");
+  this->xine->config->unregister_callback(this->xine->config,
+		  			  "effects.goom.width");
+  this->xine->config->unregister_callback(this->xine->config,
+		  			  "effects.goom.height");
+  this->xine->config->unregister_callback(this->xine->config,
+		  			  "effects.goom.csc_method");
+
   free(class_gen);
 }
 
@@ -375,6 +386,7 @@ static int goom_port_open(xine_audio_port_t *port_gen, xine_stream_t *stream,
   this->sample_rate = rate;
   this->samples_per_frame = rate / this->fps;
   this->data_idx = 0;
+  this->sample_counter = 0;
   init_yuv_planes(&this->yuv, this->width, this->height);
   this->skip_frame = 0;
   
@@ -434,7 +446,6 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
   buf = &this->buf; 
 
   this->sample_counter += buf->num_frames;
-  
   j = (this->channels >= 2) ? 1 : 0;
 
   do {
@@ -444,7 +455,7 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
       data8 += samples_used * this->channels;
   
       /* scale 8 bit data to 16 bits and convert to signed as well */
-      for( i = 0; i < buf->num_frames && this->data_idx < NUMSAMPLES;
+      for( i = samples_used; i < buf->num_frames && this->data_idx < NUMSAMPLES;
            i++, this->data_idx++, data8 += this->channels ) {
         this->data[0][this->data_idx] = ((int16_t)data8[0] << 8) - 0x8000;
         this->data[1][this->data_idx] = ((int16_t)data8[j] << 8) - 0x8000;
@@ -453,16 +464,15 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
       data = buf->mem;
       data += samples_used * this->channels;
   
-      for( i = 0; i < buf->num_frames && this->data_idx < NUMSAMPLES;
+      for( i = samples_used; i < buf->num_frames && this->data_idx < NUMSAMPLES;
            i++, this->data_idx++, data += this->channels ) {
         this->data[0][this->data_idx] = data[0];
         this->data[1][this->data_idx] = data[j];
       }
     }
   
-    if( this->sample_counter >= this->samples_per_frame &&
-        this->data_idx == NUMSAMPLES ) {
-      this->data_idx = 0;
+    if( this->sample_counter >= this->samples_per_frame ) {
+    
       samples_used += this->samples_per_frame;
 
       frame = this->vo_port->get_frame (this->vo_port, this->width_back, this->height_back,
@@ -470,14 +480,23 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
                                         VO_BOTH_FIELDS);
       
       frame->extra_info->invalid = 1;
-      frame->bad_frame = 0;
+      
+      /* frame is marked as bad if we don't have enough samples for 
+       * updating the viz plugin (calculations may be skipped).
+       * we must keep the framerate though. */
+      if( this->data_idx == NUMSAMPLES ) {
+        frame->bad_frame = 0;
+        this->data_idx = 0;
+      } else {
+        frame->bad_frame = 1;
+      }
       frame->duration = 90000 * this->samples_per_frame / this->sample_rate;
       frame->pts = pts;
       this->metronom->got_video_frame(this->metronom, frame);
       
       this->sample_counter -= this->samples_per_frame;
 
-      if (!this->skip_frame) {
+      if (!this->skip_frame && !frame->bad_frame) {
         /* Try to be fast */
         goom_frame = (uint8_t *)goom_update (this->goom, this->data, 0, 0, NULL, NULL);
 
