@@ -109,21 +109,21 @@
 #endif
 
 typedef struct osd_fontchar_s {
+  uint8_t *bmp;
   uint16_t code;
   uint16_t width;
   uint16_t height;
-  uint8_t *bmp;
 } osd_fontchar_t;
 
 struct osd_font_s {
   char             name[40];
   char            *filename;
+  osd_fontchar_t  *fontchar;
+  osd_font_t      *next;
   uint16_t         version;
   uint16_t         size;
   uint16_t         num_fontchars;
-  osd_fontchar_t  *fontchar;
-  osd_font_t      *next;
-  int              loaded;
+  uint16_t         loaded;
 }; 
 
 #ifdef HAVE_FT2
@@ -200,7 +200,7 @@ static int _osd_show (osd_object_t *osd, int64_t vpts, int unscaled ) {
   int x, y, required;
   uint8_t *c;
 
-  lprintf("osd=%p vpts=%lld\n", osd, vpts);
+  lprintf("osd=%p vpts=%"PRId64"\n", osd, vpts);
   
   this->stream->xine->port_ticket->acquire(this->stream->xine->port_ticket, 1);
   
@@ -347,7 +347,7 @@ static int _osd_hide (osd_object_t *osd, int64_t vpts) {
   osd_renderer_t *this = osd->renderer;
   video_overlay_manager_t *ovl_manager;
   
-  lprintf("osd=%p vpts=%lld\n",osd, vpts);
+  lprintf("osd=%p vpts=%"PRId64"\n",osd, vpts);
       
   if( osd->handle < 0 )
     return 0;
@@ -1481,7 +1481,16 @@ static void osd_free_object (osd_object_t *osd_to_close) {
   while( osd ) {
     if ( osd == osd_to_close ) {
       free( osd->area );
-      if( osd->ft2 ) free( osd->ft2 );
+
+#ifdef HAVE_FT2
+      if( osd->ft2 ) {
+	if ( osd->ft2->library )
+	  FT_Done_FreeType(osd->ft2->library);
+
+	free( osd->ft2 );
+      }
+#endif
+
       osd_free_encoding(osd);
       
       if( last )
