@@ -247,8 +247,6 @@ static void * sse_memcpy(void * to, const void * from, size_t len)
     /* since movntq is weakly-ordered, a "sfence"
      * is needed to become ordered again. */
     __asm__ __volatile__ ("sfence":::"memory");
-    /* enables to use FPU */
-    __asm__ __volatile__ ("emms":::"memory");
   }
   /*
    *	Now do the tail of the block
@@ -395,7 +393,7 @@ static struct {
 {
   { NULL, NULL, 0, 0 },
   { "libc memcpy()", memcpy, 0, 0 },
-#if defined(ARCH_X86) && !defined(_MSC_VER)
+#if (defined(ARCH_X86) || defined(ARCH_X86_64)) && !defined(_MSC_VER)
   { "linux kernel memcpy()", linux_kernel_memcpy, 0, 0 },
   { "MMX optimized memcpy()", mmx_memcpy, 0, MM_MMX },
   { "MMXEXT optimized memcpy()", mmx2_memcpy, 0, MM_MMXEXT },
@@ -408,10 +406,10 @@ static struct {
   { NULL, NULL, 0, 0 }
 };
 
-#if defined(ARCH_X86) && defined(HAVE_SYS_TIMES_H)
-static unsigned long long int rdtsc(int config_flags)
+#if (defined(ARCH_X86) || defined(ARCH_X86_64)) && defined(HAVE_SYS_TIMES_H)
+static int64_t rdtsc(int config_flags)
 {
-  unsigned long long int x;
+  int64_t x;
 
   /* that should prevent us from trying cpuid with old cpus */
   if( config_flags & MM_MMX ) {
@@ -463,9 +461,9 @@ void xine_probe_fast_memcpy(xine_t *xine)
   char             *buf1, *buf2;
   int               i, j, best;
   int               config_flags = -1;
-  static char      *memcpy_methods[] = {
+  static const char *memcpy_methods[] = {
     "probe", "libc",
-#if defined(ARCH_X86) && !defined(_MSC_VER)
+#if (defined(ARCH_X86) || defined(ARCH_X86_64)) && !defined(_MSC_VER)
     "kernel", "mmx", "mmxext", "sse",
 #endif
 #if defined (ARCH_PPC) && !defined (HOST_OS_DARWIN)
@@ -526,7 +524,7 @@ void xine_probe_fast_memcpy(xine_t *xine)
     t = rdtsc(config_flags) - t;
     memcpy_method[i].time = t;
 
-    xprintf(xine, XINE_VERBOSITY_LOG, "\t%s : %lld\n", memcpy_method[i].name, t);
+    xprintf(xine, XINE_VERBOSITY_LOG, "\t%s : %" PRIu64 "\n", memcpy_method[i].name, t);
 
     if( best == 0 || t < memcpy_method[best].time )
       best = i;
